"""Shim for jupyter packaging migration."""
from __future__ import annotations

import os
import sys
from pathlib import Path
from typing import Any

import tomli_w  # type:ignore[import-not-found]

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib

__this_shim = sys.modules.pop("jupyter_packaging")
__current_directory = sys.path.pop(0)

import jupyter_packaging as __real_jupyter_packaging  # type:ignore[import-not-found]

sys.path.insert(0, __current_directory)
sys.modules["jupyter_packaging"] = __this_shim


def _write_config(path: Any, data: Any) -> None:
    pyproject = Path("pyproject.toml")
    top = tomllib.loads(pyproject.read_text(encoding="utf-8"))
    current = top
    parts = path.split(".")
    for part in parts[:-1]:
        if part in current:
            current = current[part]
        else:
            current[part] = current = {}
    if parts[-1] in current:
        existing = current[parts[-1]]
    else:
        existing = current[parts[-1]] = {}
    existing.update(data)
    pyproject.write_text(tomli_w.dumps(top), encoding="utf-8")


_npm_kwargs = ["path", "build_dir", "source_dir", "build_cmd", "npm"]


def _normalize_path(path: str) -> str:
    path = str(path)
    cwd = os.getcwd()  # noqa: PTH109
    if path.startswith(cwd):
        return os.path.relpath(path, cwd)
    return path


def _get_build_kwargs(**kwargs: Any) -> dict[str, Any]:
    build_kwargs = {}
    for name in _npm_kwargs:
        value = kwargs[name]
        if value is not None:
            if name in ["path", "build_dir", "source_dir"]:
                value = _normalize_path(value)
            build_kwargs[name] = value
    if kwargs.get("force"):
        build_kwargs["force"] = True
    return build_kwargs


def skip_if_exists(paths: list[str], *args: str) -> Any:
    """Shim for skip if exists"""
    if paths:
        data = {"skip-if-exists": [_normalize_path(p) for p in paths]}
        _write_config("tool.hatch.build.hooks.jupyter-builder", data)
    return __real_jupyter_packaging.skip_if_exists(paths, *args)


def ensure_targets(targets: list[str]) -> Any:
    """Shim for ensure targets"""
    if targets:
        data = {"ensured-targets": [_normalize_path(t) for t in targets]}
        _write_config("tool.hatch.build.hooks.jupyter-builder", data)
    return __real_jupyter_packaging.ensure_targets(targets)


def wrap_installers(
    pre_develop: Any = None,
    pre_dist: Any = None,
    post_develop: Any = None,
    post_dist: Any = None,
    ensured_targets: Any = None,
    skip_if_exists: Any = None,
) -> Any:
    """Shim for wrap_installers."""
    if pre_develop or post_develop:
        func = pre_develop or post_develop
        build_kwargs = _get_build_kwargs(**func.__kwargs)
        _write_config("tool.hatch.build.hooks.jupyter-builder.editable-build-kwargs", build_kwargs)

    if pre_dist or post_dist:
        func = pre_dist or post_dist
        build_kwargs = _get_build_kwargs(**func.__kwargs)
        _write_config("tool.hatch.build.hooks.jupyter-builder.build-kwargs", build_kwargs)

    if skip_if_exists:
        data = {"skip-if-exists": [_normalize_path(p) for p in skip_if_exists]}
        _write_config("tool.hatch.build.hooks.jupyter-builder", data)

    return __real_jupyter_packaging.wrap_installers(
        pre_develop=pre_develop,
        pre_dist=pre_dist,
        post_develop=post_develop,
        post_dist=post_dist,
        ensured_targets=ensured_targets,
        skip_if_exists=skip_if_exists,
    )


def create_cmdclass(
    prerelease_cmd: Any = None,
    package_data_spec: Any = None,
    data_files_spec: Any = None,
    exclude: Any = None,
) -> Any:
    """Shim for create_cmdclass."""
    shared_data = {}
    if data_files_spec is not None:
        for path, dname, pattern in data_files_spec:
            if Path(dname).is_absolute():
                dname = os.path.relpath(dname, os.getcwd())  # noqa: PTH109, PLW2901
            if pattern == "**":
                shared_data[dname] = path
            else:
                shared_data[f"{dname}/{pattern}"] = f"{path}/{pattern}"

    _write_config("tool.hatch.build.targets.wheel.shared-data", shared_data)

    return __real_jupyter_packaging.create_cmdclass(
        prerelease_cmd=prerelease_cmd,
        package_data_spec=package_data_spec,
        data_files_spec=data_files_spec,
        exclude=exclude,
    )


def install_npm(
    path: Any = None,
    build_dir: Any = None,
    source_dir: Any = None,
    build_cmd: str = "build",
    force: bool = False,
    npm: Any = None,
) -> Any:
    """Shim for install_npm."""
    build_kwargs = _get_build_kwargs(**locals())
    if build_kwargs:
        _write_config("tool.hatch.build.hooks.jupyter-builder.build-kwargs", build_kwargs)

    return __real_jupyter_packaging.install_npm(
        path=path,
        build_dir=build_dir,
        source_dir=source_dir,
        build_cmd=build_cmd,
        force=force,
        npm=npm,
    )


def npm_builder(
    path: Any = None,
    build_dir: Any = None,
    source_dir: Any = None,
    build_cmd: str = "build",
    force: bool = False,
    npm: Any = None,
) -> Any:
    """Shim for npm_builder."""
    func = __real_jupyter_packaging.npm_builder(
        path=path,
        build_dir=build_dir,
        source_dir=source_dir,
        build_cmd=build_cmd,
        force=force,
        npm=npm,
    )
    func.__kwargs = {}
    for name in [*_npm_kwargs, "force"]:
        func.__kwargs[name] = locals()[name]
    return func


def __getattr__(name: str) -> Any:
    """Defer to the original for all others."""
    return getattr(__real_jupyter_packaging, name)


del __this_shim
del __current_directory



"""The main plugin for hatch_jupyter_builder."""
from __future__ import annotations

import os
import typing as t
import warnings
from dataclasses import dataclass, field, fields

from hatchling.builders.config import BuilderConfig
from hatchling.builders.hooks.plugin.interface import BuildHookInterface

from .utils import (
    _get_log,
    ensure_targets,
    get_build_func,
    install_pre_commit_hook,
    normalize_kwargs,
    should_skip,
)


@dataclass
class JupyterBuildConfig(BuilderConfig):
    """Build config values for Hatch Jupyter Builder."""

    install_pre_commit_hook: str = ""
    build_function: str | None = None
    build_kwargs: t.Mapping[str, str] = field(default_factory=dict)
    editable_build_kwargs: t.Mapping[str, str] = field(default_factory=dict)
    ensured_targets: list[str] = field(default_factory=list)
    skip_if_exists: list[str] = field(default_factory=list)
    optional_editable_build: str = ""


class JupyterBuildHook(BuildHookInterface[JupyterBuildConfig]):
    """The hatch jupyter builder build hook."""

    PLUGIN_NAME = "jupyter-builder"
    _skipped = False

    def initialize(self, version: str, _: dict[str, t.Any]) -> None:
        """Initialize the plugin."""
        self._skipped = False
        log = _get_log()
        log.info("Running jupyter-builder")
        if self.target_name not in ["wheel", "sdist"]:
            log.info("ignoring target name %s", self.target_name)
            self._skipped = True
            return

        if os.getenv("SKIP_JUPYTER_BUILDER"):
            log.info("Skipping the build hook since SKIP_JUPYTER_BUILDER was set")
            self._skipped = True
            return

        kwargs = normalize_kwargs(self.config)
        available_fields = [f.name for f in fields(JupyterBuildConfig)]
        for key in list(kwargs):
            if key not in available_fields:
                del kwargs[key]
        config = JupyterBuildConfig(**kwargs)

        should_install_hook = config.install_pre_commit_hook.lower() == "true"

        if version == "editable" and should_install_hook:
            install_pre_commit_hook()

        build_kwargs = config.build_kwargs
        if version == "editable":
            build_kwargs = config.editable_build_kwargs or build_kwargs

        should_skip_build = False
        if not config.build_function:
            log.warning("No build function found")
            should_skip_build = True

        elif config.skip_if_exists and version == "standard":
            should_skip_build = should_skip(config.skip_if_exists)
            if should_skip_build:
                log.info("Skip-if-exists file(s) found")

        # Get build function and call it with normalized parameter names.
        if not should_skip_build and config.build_function:
            build_func = get_build_func(config.build_function)
            build_kwargs = normalize_kwargs(build_kwargs)
            log.info("Building with %s", config.build_function)
            log.info("With kwargs: %s", build_kwargs)
            try:
                build_func(self.target_name, version, **build_kwargs)
            except Exception as e:
                if version == "editable" and config.optional_editable_build.lower() == "true":
                    warnings.warn(f"Encountered build error:\n{e}", stacklevel=2)
                else:
                    raise e
        else:
            log.info("Skipping build")

        # Ensure targets in distributable dists.
        if version == "standard":
            ensure_targets(config.ensured_targets)

        log.info("Finished running jupyter-builder")
        return



"""Compare the dist file created by a migrated package to one created by the original."""
from __future__ import annotations

import argparse
import glob
import logging
import os
import shutil
import subprocess
import sys
import tarfile
import zipfile
from pathlib import Path


def build_file(dirname: str, dist_name: str) -> None:
    """Build a dist file in a directory."""
    orig_dir = Path.cwd()
    os.chdir(dirname)
    if Path("dist").exists():
        shutil.rmtree("dist")
    subprocess.check_call([sys.executable, "-m", "build", f"--{dist_name}"])
    os.chdir(orig_dir)


def get_tar_names(dirname: str) -> set[str]:
    """Get the tarball names in a directory."""
    dist_file = glob.glob(f"{dirname}/dist/*.tar.gz")[0]  # noqa: PTH207
    tarf = tarfile.open(dist_file, "r:gz")
    return set(tarf.getnames())


def get_zip_names(dirname: str) -> set[str]:
    """Get the zip (wheel) file names in a directory."""
    wheel_file = glob.glob(f"{dirname}/dist/*.whl")[0]  # noqa: PTH207
    with zipfile.ZipFile(wheel_file, "r") as f:
        return set(f.namelist())


def filter_file(path: str) -> bool:
    """Filter a file path for interesting files."""
    if "egg-info" in path:
        return True
    path_obj = Path(path)
    ext = path_obj.suffix
    if not ext:
        return True
    if path_obj.name in [path, "setup.py", "setup.cfg", "MANIFEST.in"]:
        return True
    return False


def main(source_dir: str, target_dir: str, dist_name: str) -> dict[str, list[str]]:
    """The main script."""
    subprocess.check_call([sys.executable, "-m", "pip", "install", "build"])

    logger = logging.getLogger(__name__)
    logging.basicConfig()

    build_file(source_dir, dist_name)
    build_file(target_dir, dist_name)

    if dist_name == "sdist":
        source_names = get_tar_names(source_dir)
        target_names = get_tar_names(target_dir)
    else:
        source_names = get_zip_names(source_dir)
        target_names = get_zip_names(target_dir)

    removed = list(source_names - target_names)
    removed = [r for r in removed if not filter_file(r)]
    if removed:
        logger.info("\nRemoved_files:")
        [logger.info(f) for f in removed]  # type:ignore[func-returns-value]

    added = list(target_names - source_names)
    added = [a for a in added if not filter_file(a)]
    if added:
        logger.info("\nAdded files:")
        [logger.info(f) for f in added]  # type:ignore[func-returns-value]

    logger.info("")

    return {"added": added, "removed": removed}


def make_parser(
    parser: argparse.ArgumentParser | None = None, prog: str | None = None
) -> argparse.ArgumentParser:
    """Make an arg parser."""
    if parser is None:
        parser = argparse.ArgumentParser(prog=prog)
    parser.add_argument(dest="source_dir", help="Source Directory")
    parser.add_argument(dest="target_dir", help="Target Directory")
    parser.add_argument(dest="dist_name", help="Dist name")
    return parser


def run(args: argparse.Namespace | None = None) -> None:
    """Run the cli."""
    if args is None:
        parser = make_parser(prog=f"{sys.executable} -m hatch_jupyter_builder.compare_migrated")
        args = parser.parse_args()
    main(args.source_dir, args.target_dir, args.dist_name)



"""Register hooks for the plugin."""
from typing import Type

from hatchling.plugin import hookimpl

from .plugin import JupyterBuildHook


@hookimpl
def hatch_register_build_hook() -> Type[JupyterBuildHook]:
    """Get the hook implementation."""
    return JupyterBuildHook
